#  Copyright (c) 2020 Pim Hazebroek
#  This program is made available under the terms of the MIT License.
#  For full details check the LICENSE file at the root of the project.

"""
The BME680 sensor - Temperature, Humidity, Pressure and Gas Sensor
=========================================

See https://www.adafruit.com/product/3660
"""
import logging

import adafruit_bme680
from sensors.sensor import Sensor


class BME680(Sensor):
    """
    Concrete implementation of the BME680 sensor.
    """
    def read(self):
        """
        Reads the temperature, humidity, air pressure and VOC gas.
        """
        logging.info('Reading BME680 sensor')
        sensor = adafruit_bme680.Adafruit_BME680_I2C(super().i2c)
        measurements = {
            'temperature': sensor.temperature,
            'humidity': sensor.humidity,
            'air-pressure': sensor.pressure,
            'voc-gas': sensor.gas
        }
        logging.info('Measurement: %s', measurements)
        return measurements



#  Copyright (c) 2020 Pim Hazebroek
#  This program is made available under the terms of the MIT License.
#  For full details check the LICENSE file at the root of the project.
"""Application wide constants that are shared across multiple modules"""


ONE_MB = '1048576'
LINE_SEP = '\n'
SIXTY_SECONDS = 60
MODE_READ = 'r'
MODE_WRITE = 'w'

# Supported sensors
SENSOR_BME680 = 'BME680'
SENSOR_SHT31_D = 'SHT31-D'
SENSOR_TSL2591 = 'TSL2591'

# Supported gain modes for TSL2591
GAIN_LOW = 'GAIN_LOW'
GAIN_MEDIUM = 'GAIN_MEDIUM'
GAIN_HIGH = 'GAIN_HIGH'
GAIN_MAX = 'GAIN_MAX'
DEFAULT_GAIN = GAIN_MEDIUM

# Supported integration times for TSL2591
IT_TIME_100MS = '100ms'
IT_TIME_200MS = '200ms'
IT_TIME_300MS = '300ms'
IT_TIME_400MS = '400ms'
IT_TIME_500MS = '500ms'
IT_TIME_600MS = '600ms'
DEFAULT_IT_TIME = IT_TIME_100MS



#  Copyright (c) 2020 Pim Hazebroek
#  This program is made available under the terms of the MIT License.
#  For full details check the LICENSE file at the root of the project.
"""The base for every sensor."""

import abc
import busio
from board import SCL, SDA


class Sensor(metaclass=abc.ABCMeta):
    """
    A base class for sensors connected to the station via I2C.
    """
    def __init__(self):
        self._i2c = busio.I2C(SCL, SDA)

    @property
    def i2c(self):
        """Returns the I2C instance, needed to init the concrete sensor."""
        return self._i2c

    @abc.abstractmethod
    def read(self):
        """
        Connect to the sensor over I2C and perform a measurement and return the value(s) that were measured.
        May return multiple values if the sensor measures multiple metrics.

        :rtype dict
        """
        raise NotImplementedError
